@@ -14,53 +14,67 @@
  * limitations under the License.
  *
 */
-
+//! [complete]
 #include <iostream>
-#include <ignition/math.hh>
+#include <gz/math.hh>
 
 int main(int argc, char **argv)
 {
   // Create a Vector2 called vec2 of doubles using the typedef Vector2d.
   // The initial x any y values are zero.\n\n";
-  ignition::math::Vector2d vec2;
-
   // The x and y component of vec2 can be set at anytime.
+//! [constructor]
+  gz::math::Vector2d vec2;
   vec2.Set(2.0, 4.0);
+//! [constructor]
 
   // The Vector2 class is a template, so you can also create a Vector2 using
-  // ignition::math::Vector2<double>
-  ignition::math::Vector2<double> vec2a;
+  // gz::math::Vector2<double>
+  //! [constructor2]
+  gz::math::Vector2<double> vec2a;
 
   vec2a.Set(1.0, 2.0);
+  //! [constructor2]
 
   // It's also possible to set initial values. This time we are using
   // a Vector2 of floats
-  ignition::math::Vector2f vec2b(1.2f, 3.4f);
+  //! [constructor3]
+  gz::math::Vector2f vec2b(1.2f, 3.4f);
+  //! [constructor3]
 
   // We can output the contents of each vector using std::cout
+//! [stdout]
   std::cout << "Vec2: " << vec2 << "\n"
             << "Vec2a: " << vec2a << "\n"
             << "Vec2b: " << vec2b << "\n";
+//! [stdout]
 
   // You can also get access to each component in the vector using the
   // X(), Y() accessors or the [] operator.
+  //! [access]
   std::cout << "Vec2: x=" << vec2.X() << " y=" << vec2.Y() << "\n";
   std::cout << "Vec2a: x=" << vec2a[0] << " y=" << vec2a[1] << "\n";
   std::cout << "Vec2b: x=" << vec2b.X() << " y=" << vec2b[1] << "\n";
+  //! [access]
 
   // The [] operator is clamped to the range [0, 1]
   std::cout << vec2[3] << std::endl;
 
   // The Vector2 class overloads many common operators
+//! [operators]
   std::cout << vec2 * vec2a << "\n"
             << vec2 + vec2a << "\n"
             << vec2 - vec2a << "\n"
             << vec2 / vec2a << "\n";
+//! [operators]
 
   // There are also many useful function such as finding the distance
   // between two vectors
+//! [distance]
   std::cout << vec2.Distance(vec2a) << std::endl;
+//! [distance]
 
-  // There are more functions in Vector2. Take a look at the API;
-  // http://ignitionrobotics.org/libraries/ign_mat/api
+  // There are more functions in Vector2. Take a look at the API:
+  // https://ignitionrobotics.org/libs/math
 }
+//! [complete]
