@@ -16,7 +16,7 @@
 */
 #include "RotationSplinePrivate.hh"
 
-using namespace ignition;
+using namespace gz;
 using namespace math;
 
 /////////////////////////////////////////////////
